/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Xul Explorer.
 *
 * The Initial Developer of the Original Code is Mark Finkle.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Mark Finkle, <mark.finkle@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


// ------------------------------------------------------------------
// use helper functions to hook up the designer object so "this"
// works in the designer object
// ------------------------------------------------------------------

function designer_startup() {
  designer.startup();
}

function designer_shutdown() {
  designer.shutdown();
}


// ------------------------------------------------------------------
// attach to window events so designer object can startup / shutdown
// ------------------------------------------------------------------

window.addEventListener("load", designer_startup, false);
window.addEventListener("unload", designer_shutdown, false);


// ------------------------------------------------------------------
// designer object
// ------------------------------------------------------------------

var designer = {
  initialized : false,
  editor : null,
  previewInline : null,
  previewWindow : null,
  
  _initSnippets : function() {
    snippetManager.init(this, "chrome://explorer/content/snippets.json");

    var stringbundle = document.getElementById("strings_explorer");
    var defaultCategory = stringbundle.getString("category.default");
    
    var categories = snippetManager.categories;
    var categoryList = document.getElementById("snippet-type");
    categoryList.appendItem(defaultCategory, -1);
    for (var iCategory in categories)
      categoryList.appendItem(categories[iCategory], iCategory);
    categoryList.selectedIndex = 0;

    var snippets = snippetManager.snippets;
    var snippetList = document.getElementById("snippets-list");
    for (var iSnippet in snippets)
      snippetList.appendItem(snippets[iSnippet].title, iSnippet);
  },
    
  _initEditor : function() {
    this.editor.makeEditable("html", false);
  
    var textEditor = this.editor.getEditor(this.editor.contentWindow).QueryInterface(Components.interfaces.nsIPlaintextEditor);
    textEditor.enableUndo(true);
    textEditor.rootElement.style.fontFamily = "-moz-fixed";
    textEditor.rootElement.style.fontSize = "10pt";
    textEditor.rootElement.style.whiteSpace = "nowrap";
    textEditor.rootElement.style.margin = 0;

    var snippets = snippetManager.snippets;
    for (var iSnippet in snippets) {
      if (snippets[iSnippet].title == "Window")
        textEditor.insertText(snippets[iSnippet].code);
    }
  },
  
  startup : function() {
    if (this.initialized)
      return;
      
     this.initialized = true;
  
    // locate and cache elements
    this.editor = document.getElementById("source");
    this.previewInline = document.getElementById("preview");
  
    // initialize
    this._initSnippets();
    this._initEditor();

    fileController.init(this);
    editController.init(this);
    viewController.init(this);
    helpController.init(this);
    
    window.controllers.appendController(fileController);
    window.controllers.appendController(editController);
    window.controllers.appendController(viewController);
    window.controllers.appendController(helpController);
  },
  
  shutdown : function() {
  },

  clearSource : function() {
    var textEditor = this.editor.getEditor(this.editor.contentWindow).QueryInterface(Components.interfaces.nsIPlaintextEditor);
    textEditor.enableUndo(false);
    textEditor.selectAll();
    textEditor.deleteSelection(1);
    textEditor.enableUndo(true);
    textEditor.resetModificationCount();
  },
    
  getSource : function() {
    var source = this.editor.getEditor(app.editor.contentWindow).outputToString("text/plain", "none");
    return source;
  },
  
  setSource : function(aText) {
    this.clearSource();
    this.insertText(aText);
  },

  insertText : function(aText) {
    var textEditor = this.editor.getEditor(this.editor.contentWindow).QueryInterface(Components.interfaces.nsIPlaintextEditor);
    textEditor.insertText(aText);
  },

  insertSnippet : function() {
    var snippetList = document.getElementById("snippets-list");
    if (snippetList.selectedItem) {
      snippetManager.insert(snippetList.selectedItem.value);  
    }
  },
  
  refreshPreview : function() {
    var source = this.getSource();
    var dataURI = "data:application/vnd.mozilla.xul+xml," + encodeURIComponent(source);
    
    this.previewInline.setAttribute("src", dataURI);
    
    if (this.previewWindow != null && this.previewWindow.closed == false) {
      this.previewWindow.document.location = dataURI;
    }
  },
  
  openPreview : function() {
    if (this.previewWindow == null || this.previewWindow.closed == true) {
      this.previewWindow = window.open("about:blank", "explorer-preview", "chrome,dependent,all,centerscreen,resizable=yes,width=400,height=400");
    }
    return this.previewWindow;
  },
  
  getHelpURI : function(cmd) {
    var stringbundle = document.getElementById("strings_explorer");
    var helpURI;
    switch (cmd) {
      case "cmd_help": 
        helpURI = stringbundle.getString("appReference.url");
        break;
      case "cmd_xulref": 
        helpURI = stringbundle.getString("xulReference.url");
        break;
      case "cmd_xultut": 
        helpURI = stringbundle.getString("xulTutorial.url");
        break;
      case "cmd_jsref": 
        helpURI = stringbundle.getString("jsReference.url");
        break;
      case "cmd_keyword": 
      {
        var keyword = this.editor.contentWindow.getSelection().toString();
        if (keyword.length > 0)
          helpURI = stringbundle.getString("xulKeyword.url") + keyword;
        else
          helpURI = stringbundle.getString("xulReference.url");
        break;
      }
    }
    return helpURI;
  },
  
  getEditor : function() {
    var textEditor = this.editor.getEditor(this.editor.contentWindow).QueryInterface(Components.interfaces.nsIPlaintextEditor);
    return textEditor;
  }
};


// ------------------------------------------------------------------
// helper methods to handle snippets
// TODO: move to an object
// ------------------------------------------------------------------

function doInsertSnippet() {
  designer.insertSnippet();
}

function doSnippetCateorgyChange() {
    var categoryList = document.getElementById("snippet-type");
    if (categoryList.selectedItem) {
      var snippets = snippetManager.filterByCategory(categoryList.selectedItem.value);

      var snippetList = document.getElementById("snippets-list");
      
      while (snippetList.getRowCount() > 0)
        snippetList.removeItemAt(0);
        
      for (var iSnippet in snippets)
        snippetList.appendItem(snippets[iSnippet].title, iSnippet);
    }
}